/*
 * ----------------------------------------------------------------------------
 *
 * Gm3u  - A MP3 playlist editor
 *
 * (c) 2000 by Bodo Bauer
 *
 * ----------------------------------------------------------------------------
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ----------------------------------------------------------------------------
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>
#include <linux/limits.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "filetree.h"
#include "id3.h"

static void UnfoldDirectory ( char *path, GtkWidget *root );
static gboolean IsDirectory ( char* path );
static void ExpandTreeCB ( GtkTreeItem *treeitem, gchar *subdir );
static void DirSelectCB  ( GtkTreeItem *treeitem, gchar *subdir );
static void FileSelectCB ( GtkTreeItem *treeitem, gpointer user_data );
static void FileUnSelectCB ( GtkTreeItem *treeitem, gpointer user_data );

static char *LastSelection;
static GList *Selection = NULL;

/*
 *  Create the initial tree
 */
void InitTree ( GtkWidget *gm3u ) {
  GtkWidget *tree = (GtkWidget*) gtk_object_get_data ( GTK_OBJECT ( gm3u ), "file_tree" );
  GtkWidget *root;
  GtkWidget *subtree;
  
  /* create root item */
  root = gtk_tree_item_new_with_label ( "/" );
  gtk_tree_append ( GTK_TREE ( tree ), root );
  subtree = gtk_tree_new ();
  gtk_tree_item_set_subtree ( GTK_TREE_ITEM ( root ), subtree );
  UnfoldDirectory ( "/", subtree ); 
  gtk_tree_item_expand ( GTK_TREE_ITEM ( root ) );
  gtk_object_set_data ( GTK_OBJECT ( tree ), "file_selection_list", &Selection );
  gtk_widget_show ( root );
}

/*
 * Unfold subdirectory 'path' and present it in the tree starting at 'root' node
 */
static void UnfoldDirectory ( char *path, GtkWidget *root ) {
  DIR           *dir;
  struct dirent *direntry;
  gchar         buffer[PATH_MAX];
  GtkWidget     *item;
  GtkWidget     *subtree;

  if ( ( dir = opendir ( path ) ) ) {
    while ( ( direntry = readdir ( dir ) ) ) {
      /* ignore . and .. */
      if ( ! ( !strcmp ( direntry->d_name, ".." ) || !strcmp ( direntry->d_name, "." )  ) ) { 
	/* get full path */
	sprintf ( buffer,  "%s/%s", path, direntry->d_name );
	if ( IsDirectory ( buffer ) ) {
	  item = gtk_tree_item_new_with_label ( direntry->d_name );
	  gtk_tree_append ( GTK_TREE ( root ), item );
	  subtree = gtk_tree_new ();
	  gtk_tree_item_set_subtree ( GTK_TREE_ITEM ( item ), subtree );
	  gtk_signal_connect ( GTK_OBJECT ( item ), "expand", GTK_SIGNAL_FUNC ( ExpandTreeCB ), g_strdup ( buffer ) );
	  gtk_signal_connect ( GTK_OBJECT ( item ), "select", GTK_SIGNAL_FUNC ( DirSelectCB ),  g_strdup ( buffer ) );
	  gtk_widget_show ( item );
	} else {
	  TreeItem *item = g_malloc0 ( sizeof ( TreeItem ) );
	  item->widget = gtk_tree_item_new_with_label ( direntry->d_name );
	  item->filename = g_strdup ( buffer );
	  gtk_signal_connect ( GTK_OBJECT ( item->widget ), "select", GTK_SIGNAL_FUNC ( FileSelectCB ),     item );
	  gtk_signal_connect ( GTK_OBJECT ( item->widget ), "deselect", GTK_SIGNAL_FUNC ( FileUnSelectCB ), item );
	  gtk_tree_append ( GTK_TREE ( root ), item->widget );
	  gtk_widget_show ( item->widget );
	}
      } 
    }
    closedir ( dir );
  }
}

/*
 * Check if 'path' is a directory
 */
static gboolean IsDirectory ( gchar *path ) {
  struct stat buf;
  return ( ( stat ( path, &buf ) < 0 ) ? FALSE : ( S_ISDIR ( buf.st_mode ) ) );
}
  
/*
 * Callback to expand a subdirectory  node
 */
static void ExpandTreeCB ( GtkTreeItem *treeitem, gchar *subdir ) {
  /* show what's in this subdir */
  UnfoldDirectory ( subdir, GTK_WIDGET ( GTK_TREE_ITEM_SUBTREE ( treeitem ) ) );
  /* but don't unfold it again... */
  gtk_signal_disconnect_by_func ( GTK_OBJECT ( treeitem ), GTK_SIGNAL_FUNC ( ExpandTreeCB ), subdir );
  g_free ( subdir );
}

/* 
 * Directory selected -> unselect it again, this is not what we want
 */
void DirSelectCB  ( GtkTreeItem *tree_item, gchar *subdir ) {
  gtk_tree_item_deselect ( tree_item );

}

/*
 * File Selected
 */
void FileSelectCB ( GtkTreeItem *treeitem, gpointer user_data ) {
  TreeItem *item = (TreeItem*) user_data;

  if ( gnome_config_get_bool ( "gm3u/id3_tags/show=FALSE" )  ) {
    DisplayID3Tag ( item->filename );
  }
  if ( LastSelection ) {
    g_free ( LastSelection );
  }
  LastSelection = g_strdup ( item->filename );
  Selection = g_list_append ( Selection, item );
}

/*
 * File Deselected
 */
void FileUnSelectCB ( GtkTreeItem *treeitem, gpointer user_data ) {
  TreeItem *item = (TreeItem*)user_data;

  ClearID3Tag();  
  Selection = g_list_remove ( Selection, item );
}

/*
 * Play selected file
 */
void on_play_mp3_button_clicked ( GtkButton *button, gpointer user_data ) {
  if ( LastSelection ) {
    gchar *buffer = g_malloc0 ( sizeof ( gchar ) * PATH_MAX * 2 );
    sprintf (buffer,  "%s %s", gnome_config_get_string ( "gm3u/path/mp3_player=xmms" ), LastSelection );
    gnome_execute_shell ( gnome_util_user_home(), buffer );
    g_free ( buffer );
  }
}



















