/*
 * ----------------------------------------------------------------------------
 *
 * Gm3u  - A MP3 playlist editor
 *
 * (c) 2000 by Bodo Bauer
 *
 * ----------------------------------------------------------------------------
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ----------------------------------------------------------------------------
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <sys/stat.h>
#include <unistd.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "filetree.h"
#include "fileselect.h"
#include "id3.h"

/* exported stuff */
void CheckForChanges ( void func ( void ) );
gboolean unsaved_changes = FALSE;
void InitPlaylist ( GtkWidget* );

typedef struct _PlayListItem {
  gchar     *filename;
  gchar     *basename;
  GtkWidget *listitem;
  off_t     size;
} PlayListItem;


/* local prototypes */
static gchar     *playlist_file = NULL;
static GList     *playlist      = NULL;
static GtkWidget *gm3u          = NULL;

static void New ( void );
static void Load ( void );
static void Save ( void );
static void SaveAs ( void );
static void Restore ( void );
static void Clear ( void );
static void LoadPlaylist ( gchar *filename );
static void WritePlaylist ( gchar *filename );
static void AddSongToPlaylist ( gchar *filename );

/*
 *  Get prepared...
 */
void InitPlaylist ( GtkWidget *dialog ) {
  gm3u = dialog;
}

/*
 * Load playlist from File
 */
void LoadPlaylist ( gchar *filename ) {
  if ( filename ) {
    FILE *fd;

    /* clean up before we load the new playlist */
    Clear ();
    if ( playlist_file ) {
      g_free ( playlist_file );
    }
    playlist_file = filename;

    if ( ( fd =fopen ( filename, "r" ) ) ) {
      gchar buffer[PATH_MAX];
      
      while ( fgets ( buffer, PATH_MAX, fd ) ) {
	buffer[strlen(buffer)-1] = (gchar) 0;
	AddSongToPlaylist ( buffer );
      }
      fclose ( fd );
    }
  } 
}

/*
 * Write playlist to File
 */
void WritePlaylist ( gchar *filename ) {
  if ( filename ) {
    FILE *fd;

    if ( playlist_file && playlist_file != filename ) {
      g_free ( playlist_file );
    }
    playlist_file = filename;

    if ( ( fd = fopen ( playlist_file, "w" ) ) ) {
      GList *node = playlist;

      while ( node ) {
	fprintf ( fd, "%s\n", ((PlayListItem*)(node->data))->filename );
	node = node->next;
      }
      fclose ( fd );
    }
    unsaved_changes = FALSE;
  } 
}
/*
 * popup warning dialog and call given function only when user
 * agrees to doscard changes 
 */
void CheckForChanges ( void func ( void ) ) {
  if ( unsaved_changes ) {
    GtkWidget *dialog = create_unsaved_changes_dialog ();
    gtk_object_set_data ( GTK_OBJECT (dialog), "call_me_if_ok", func );
    gtk_widget_show ( dialog );
  } 
}

/*
 * New playlist
 */
void New ( void ) {
  if ( unsaved_changes ) {
    CheckForChanges ( &New );
  } else {
    if ( playlist_file ) {
      g_free ( playlist_file );
    }
    Clear();
    playlist_file = NULL;
  }
}

/*
 * Load playlist
 */
void Load ( void ) {
  if ( unsaved_changes ) {
    CheckForChanges ( &Load );
  } else {
    SelectFile ( &LoadPlaylist );
  }
}

/*
 * Save Playlist
 */
void Save ( void ) {
  if ( playlist_file ) {
    WritePlaylist ( playlist_file );
  } else {
    SaveAs();
  }
}

/*
 * Save Playlist in new file
 */
void SaveAs ( void ) {
  SelectFile ( &WritePlaylist );
}

/*
 * Restore Playlist
 */
void Restore ( void ) {
  gchar *filename = NULL;

  if ( unsaved_changes ) {
    CheckForChanges ( &Restore );
  } else {
    if ( playlist_file ) {
      filename = g_strdup ( playlist_file );
    }
    Clear ();
    if ( filename ) {
      LoadPlaylist ( filename );
    }
  }
}

/*
 * Remove all items in playlist
 */
void Clear ( void ) {
  if ( unsaved_changes ) {
    CheckForChanges ( &Clear );
  } else {
    if ( playlist ) {
      GList *node = playlist;
      GtkWidget *list = gtk_object_get_data ( GTK_OBJECT ( gm3u ), "play_list" );
      while ( node ) {
	PlayListItem *item = (PlayListItem*)(node->data);
	gtk_container_remove ( GTK_CONTAINER ( list ), item->listitem );
	if ( item->filename ) {
	  g_free ( item->filename );
	}
	g_free ( item );
	node->data = NULL;
	node = node->next;
      }
      g_list_free ( playlist );
      playlist = NULL;
    }
  }
}

/* 
 * add song to playlist
 */
void AddSongToPlaylist ( gchar *filename ) {
  GtkWidget *list = gtk_object_get_data ( GTK_OBJECT ( gm3u ), "play_list" );
  struct stat buf;

  if ( ! stat ( filename, &buf ) ) {
    PlayListItem *playlistitem = (PlayListItem*) g_malloc0 ( sizeof ( PlayListItem ) );

    playlistitem->size     = buf.st_size;
    playlistitem->filename = g_strdup ( filename );
    playlistitem->basename = g_basename ( playlistitem->filename ); 
    playlist = g_list_append ( playlist, playlistitem );
    playlistitem->listitem = gtk_list_item_new_with_label ( playlistitem->basename );
    gtk_signal_connect ( GTK_OBJECT (  playlistitem->listitem ), "deselect", GTK_SIGNAL_FUNC ( on_playlist_item_unselect ), list );
    gtk_signal_connect ( GTK_OBJECT (  playlistitem->listitem ), "select", GTK_SIGNAL_FUNC ( on_playlist_item_select ), list );
    gtk_object_set_data ( GTK_OBJECT ( playlistitem->listitem ), "playlistitem", playlistitem );
    gtk_container_add ( GTK_CONTAINER ( list ), playlistitem->listitem );
    gtk_widget_show ( playlistitem->listitem );
  }
}

/*
 * Exchange two items in playlist
 */
void Exchange ( PlayListItem *itemA,  PlayListItem *itemB ) {
  GList     *listentryA, *listentryB;
  GtkWidget *widgetA, *widgetB;

  listentryA = g_list_find ( playlist, itemA );
  listentryB = g_list_find ( playlist, itemB );
  widgetA    = itemA->listitem;
  widgetB    = itemB->listitem;

  if ( listentryA && listentryB ) {
    listentryA->data = (gpointer) itemB;
    listentryB->data = (gpointer) itemA;
    itemA->listitem  = widgetB;
    itemB->listitem  = widgetA;
    gtk_label_set_text  ( GTK_LABEL ((GTK_BIN ( widgetA ))->child), itemB->basename );
    gtk_label_set_text  ( GTK_LABEL ((GTK_BIN ( widgetB ))->child), itemA->basename );
    gtk_object_set_data ( GTK_OBJECT ( widgetA ), "playlistitem", itemB );
    gtk_object_set_data ( GTK_OBJECT ( widgetB ), "playlistitem", itemA );
  }
}

/* ---------------------------------------------------------------------
 * Callbacks
 * --------------------------------------------------------------------- */

/*
 * Item selected in playlist -> store pointer 
 */
void on_playlist_item_select (GtkWidget *widget,  gpointer user_data ) {
  PlayListItem *item = (PlayListItem*) gtk_object_get_data ( GTK_OBJECT ( widget ), "playlistitem" );
  GtkWidget    *list = (GtkWidget*) user_data;
  GnomeAppBar *status_bar = (GnomeAppBar*) gtk_object_get_data ( GTK_OBJECT ( gm3u ), "status_bar" );

  gnome_appbar_set_status ( status_bar, "" );
  gtk_object_set_data ( GTK_OBJECT ( list ), "selection", item );
  DisplayID3Tag ( item->filename );
}

/*
 * Item unselected in playlist -> clear pointer 
 */
void on_playlist_item_unselect (GtkWidget *widget,  gpointer user_data ) {
  PlayListItem *selection = (PlayListItem*) gtk_object_get_data ( GTK_OBJECT ( user_data ), "selection" );  
  PlayListItem *item = (PlayListItem*) gtk_object_get_data ( GTK_OBJECT ( widget ), "playlistitem" );
  GnomeAppBar *status_bar = (GnomeAppBar*) gtk_object_get_data ( GTK_OBJECT ( gm3u ), "status_bar" );

  gnome_appbar_set_status ( status_bar, "" );  
  if ( selection == item ) {
    gtk_object_set_data ( GTK_OBJECT ( user_data ), "selection", NULL );
    ClearID3Tag ();
  }
}

/* 
 * Menu Open
 */
void on_open1_activate ( GtkMenuItem *menuitem, gpointer user_data ) {
  Load ();
}

/*
 * Menu New
 */
void on_new1_activate ( GtkMenuItem *menuitem, gpointer user_data ) {
  New ();
}

/*
 * Menu Save
 */
void on_save1_activate ( GtkMenuItem *menuitem, gpointer user_data ) {
  Save ();
}

/*
 * Menu SaveAs
 */
void on_save_as1_activate ( GtkMenuItem *menuitem, gpointer user_data ) {
  SaveAs ();
}

/*
 * Menu Revert
 */
void on_revert1_activate ( GtkMenuItem *menuitem, gpointer user_data ) {
  Restore();
}

/* 
 * Menu Clear
 */
void on_clear1_activate ( GtkMenuItem *menuitem, gpointer user_data ) {
  Clear ();
}

/* 
 * Button New
 */
gboolean on_new_playlist_button_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data ) {
  New ();
  return FALSE;
}

/* 
 * Button Open
 */
gboolean on_open_playlist_button_button_press_event ( GtkWidget *widget, GdkEventButton *event, gpointer user_data ) {
  Load ();
  return FALSE;
}

/* 
 * Button Save
 */
gboolean on_save_playlist_button_button_press_event ( GtkWidget *widget, GdkEventButton *event, gpointer user_data ) {
  Save ();
  return FALSE;
}

/*
 * Button Revert
 */
gboolean on_revert_playlist_button_button_press_event ( GtkWidget *widget, GdkEventButton *event, gpointer user_data ) {
  Restore ();
  return FALSE;
}

/*
 * Button Clear
 */
gboolean on_clear_playlist_button_button_press_event ( GtkWidget *widget, GdkEventButton *event, gpointer user_data ) {
  Clear ();
  return FALSE;
}

/*
 * OK button in warning dialog clicked
 */
void on_changes_OK_clicked ( GtkButton *button, gpointer dialog ) {
  void (*func)(void) = gtk_object_get_data ( GTK_OBJECT ( dialog ), "call_me_if_ok" );

  gnome_dialog_close ( GNOME_DIALOG ( dialog ) );

  /* ignore changes and go on... */
  unsaved_changes = FALSE;

  /* now do the actual job */
  (*func)();
}

/*
 * Cancel button in warning dialog clicked
 */
void on_changes_cancel_clicked ( GtkButton *button, gpointer dialog ) {
  gnome_dialog_close ( GNOME_DIALOG ( dialog ) );
}

/*
 * Move song to top of playlist
 */
gboolean on_move_song_to_top_button_button_press_event ( GtkWidget *widget, GdkEventButton *event, gpointer user_data ) {
  PlayListItem *item      = (PlayListItem*) gtk_object_get_data ( GTK_OBJECT ( user_data ), "selection" );
  GnomeAppBar *status_bar = (GnomeAppBar*) gtk_object_get_data ( GTK_OBJECT ( gm3u ), "status_bar" );

  if ( item ) {
    GList *listentry = g_list_find ( playlist, item );

    if ( listentry && playlist != listentry ) {
      PlayListItem *new_top = listentry->data;
      GList *move    = NULL;


      move = g_list_append ( move, new_top->listitem );
      playlist = g_list_remove ( playlist, new_top );
      playlist = g_list_prepend ( playlist, new_top );
      gtk_list_remove_items_no_unref ( GTK_LIST ( gtk_object_get_data ( GTK_OBJECT ( gm3u ), "play_list" ) ),  move );
      gtk_list_prepend_items ( GTK_LIST ( gtk_object_get_data ( GTK_OBJECT ( gm3u ), "play_list" ) ),  move );
      gtk_object_set_data ( GTK_OBJECT ( new_top->listitem ), "playlistitem", new_top );      
      gtk_signal_connect ( GTK_OBJECT (  new_top->listitem ), "deselect", GTK_SIGNAL_FUNC ( on_playlist_item_unselect ), 
			   GTK_LIST ( gtk_object_get_data ( GTK_OBJECT ( gm3u ), "play_list" ) )  );
      gtk_signal_connect ( GTK_OBJECT (  new_top->listitem ), "select", GTK_SIGNAL_FUNC ( on_playlist_item_select ), 
			   GTK_LIST ( gtk_object_get_data ( GTK_OBJECT ( gm3u ), "play_list" ) )  ); 
      gtk_signal_emit_by_name ( GTK_OBJECT ( new_top->listitem ), "select", NULL );
      unsaved_changes = TRUE;
    }
  } else {
    gnome_appbar_set_status ( status_bar, "Nothing selected" );
  }
  return FALSE;
}

/*
 * Move song to end of playlist
 */
gboolean on_move_song_to_bottom_button_button_press_event ( GtkWidget *widget, GdkEventButton *event, gpointer user_data ) {
  PlayListItem *item      = (PlayListItem*) gtk_object_get_data ( GTK_OBJECT ( user_data ), "selection" );
  GnomeAppBar *status_bar = (GnomeAppBar*) gtk_object_get_data ( GTK_OBJECT ( gm3u ), "status_bar" );

  if ( item ) {
    GList *listentry = g_list_find ( playlist, item );

    if ( listentry && listentry->next ) {
      PlayListItem *new_bottom = listentry->data;
      GList *move    = NULL;

      gtk_signal_emit_by_name ( GTK_OBJECT ( new_bottom->listitem ), "deselect", NULL );
      move = g_list_append ( move, new_bottom->listitem );
      playlist = g_list_remove ( playlist, new_bottom );
      playlist = g_list_append ( playlist, new_bottom );
      gtk_list_remove_items_no_unref ( GTK_LIST ( gtk_object_get_data ( GTK_OBJECT ( gm3u ), "play_list" ) ),  move );
      gtk_list_append_items ( GTK_LIST ( gtk_object_get_data ( GTK_OBJECT ( gm3u ), "play_list" ) ),  move );
      gtk_object_set_data ( GTK_OBJECT ( new_bottom->listitem ), "playlistitem", new_bottom );      
      gtk_signal_connect ( GTK_OBJECT (  new_bottom->listitem ), "deselect", GTK_SIGNAL_FUNC ( on_playlist_item_unselect ), 
			   GTK_LIST ( gtk_object_get_data ( GTK_OBJECT ( gm3u ), "play_list" ) )  );
      gtk_signal_connect ( GTK_OBJECT (  new_bottom->listitem ), "select", GTK_SIGNAL_FUNC ( on_playlist_item_select ), 
			   GTK_LIST ( gtk_object_get_data ( GTK_OBJECT ( gm3u ), "play_list" ) )  ); 
      gtk_signal_emit_by_name ( GTK_OBJECT ( new_bottom->listitem ), "select", NULL );
      unsaved_changes = TRUE;
    }
  } else {
    gnome_appbar_set_status ( status_bar, "Nothing selected" );
  }
  return FALSE;
}

/*
 * Move song one position up in playlist
 */
gboolean on_move_song_up_button_button_press_event ( GtkWidget *widget, GdkEventButton *event, gpointer user_data ) {
  PlayListItem *item = (PlayListItem*) gtk_object_get_data ( GTK_OBJECT ( user_data ), "selection" );
  GnomeAppBar *status_bar = (GnomeAppBar*) gtk_object_get_data ( GTK_OBJECT ( gm3u ), "status_bar" );

  if ( item ) {
    GList *listentry = g_list_find ( playlist, item );

    if ( listentry && listentry->prev ) {
      gtk_signal_emit_by_name ( GTK_OBJECT ( item->listitem ), "deselect", NULL );
      Exchange ( item, (PlayListItem*)(listentry->prev->data) );
      gtk_signal_emit_by_name ( GTK_OBJECT ( item->listitem ), "select", NULL );
      gnome_appbar_set_status ( status_bar, "" );
      unsaved_changes = TRUE;
    }
  } else {
    gnome_appbar_set_status ( status_bar, "Nothing selected" );
  }
  return FALSE;
}


/* 
 * Move song one position down in playlist
 */
gboolean on_move_song_down_button_button_press_event ( GtkWidget *widget, GdkEventButton *event, gpointer user_data ) {
  PlayListItem *item = (PlayListItem*) gtk_object_get_data ( GTK_OBJECT ( user_data ), "selection" );
  GnomeAppBar *status_bar = (GnomeAppBar*) gtk_object_get_data ( GTK_OBJECT ( gm3u ), "status_bar" );

  if ( item ) {
    GList *listentry = g_list_find ( playlist, item );

    if ( listentry && listentry->next ) {
      gtk_signal_emit_by_name ( GTK_OBJECT ( item->listitem ), "deselect", NULL );
      Exchange ( item, (PlayListItem*)(listentry->next->data) );
      gtk_signal_emit_by_name ( GTK_OBJECT ( item->listitem ), "select", NULL );
      unsaved_changes = TRUE;
      gnome_appbar_set_status ( status_bar, "" );
    }
  } else {
    gnome_appbar_set_status ( status_bar, "Nothing selected" );
  }
  return FALSE;
}


/*
 * Remove song from playlist
 */
gboolean on_remove_song_button_button_press_event ( GtkWidget *widget, GdkEventButton *event, gpointer list ) {
  PlayListItem *item = (PlayListItem*) gtk_object_get_data ( GTK_OBJECT ( list ), "selection" );
  GnomeAppBar *status_bar = (GnomeAppBar*) gtk_object_get_data ( GTK_OBJECT ( gm3u ), "status_bar" );

  if ( item ) {
    gtk_container_remove ( GTK_CONTAINER ( list ), item->listitem );
    playlist = g_list_remove ( playlist, item );
    
    if ( item->filename ) {
      g_free ( item->filename );
    }
    g_free ( item );
    
    gtk_object_set_data ( GTK_OBJECT ( list ), "selection", NULL );
    gnome_appbar_set_status ( status_bar, "" );
    unsaved_changes = TRUE;
  } else {
    gnome_appbar_set_status ( status_bar, "Nothing selected" );
  }

  return FALSE;
}

/*
 * Add selection from file tree to playlist
 */
void on_add_song_to_palylist_button_clicked ( GtkButton *button, gpointer user_data ) {
  GList  **file_selection = gtk_object_get_data ( GTK_OBJECT ( user_data ), "file_selection_list" );
  GnomeAppBar *status_bar = (GnomeAppBar*) gtk_object_get_data ( GTK_OBJECT ( gm3u ), "status_bar" );

  if ( *file_selection ) {
    GList *node = *file_selection;

    while ( node ) {
      GtkWidget *item =  ((TreeItem*)(node->data))->widget;
      AddSongToPlaylist ( ((TreeItem*)(node->data))->filename );
      node = node->next;
      gtk_signal_emit_by_name ( GTK_OBJECT ( item ), "deselect", NULL );
    }
    gnome_appbar_set_status ( status_bar, "" );
    unsaved_changes = TRUE;
  } else {
    gnome_appbar_set_status ( status_bar, "Nothing selected" );
  }
}






















